<?php

/**
 * Sends requests to the server returns error if a timeout occurs.
 */
class ImlApi {

  const maxFileReadingStringLength = 4096;

  /**
   * @var string
   *  Well-formed server url to connect with
   */
  private $serverUrl;

  /**
   * @var int
   *  Timeout for server communications in seconds.
   */
  private $timeout;

  /**
   * @var string
   *  Api login.
   */
  private $login;

  /**
   * @var string
   *  Api password.
   */
  private $password;

  /**
   * @var boolean
   *  If working in test mode.
   */
  private $is_test_mode;

  /**
   * @param string $server_url
   *  Server url.
   * @param int $timeout
   *  Timeout in seconds.
   * @param string $login
   *  Api login.
   * @param string $password
   *  Api password.
   * @param boolean $is_test_mode
   *  Is test mode.
   */
  public function __construct($server_url, $timeout, $login, $password, $is_test_mode) {
    $this->serverUrl = $server_url;
    $this->timeout = $timeout;
    $this->login = $login;
    $this->password = $password;
    $this->is_test_mode = $is_test_mode;
  }

  /**
   * @param array $parameters
   * @return array
   */
  public function getOrderStatuses(array $parameters) {
    return $this->sendRequest('GetStatuses', $parameters);
  }

  /**
   * @param array $parameters
   * @return array
   */
  public function saveOrder(array $parameters) {
    return $this->sendRequest('CreateOrder', $parameters);
  }
  
  /**
   * @param string $url_suffix
   * @param array $parameters
   * 
   * @return array
   *  Resulted data structure representing decoded xml or containing error information.
   */
  private function sendRequest($url_suffix, array $parameters) {
    if ($this->is_test_mode) {
      $parameters['Test'] = 'True';
    }
    
    try {
      return json_decode($this->doPostRequest($url_suffix, $parameters), TRUE);
    }
    catch (Exception $exception) {
      return array('Errors' => array($exception->getMessage()));
    }
  }

  /**
   * @param int $communication_start_time
   *
   * @return bool
   */
  private function isTimedOut($communication_start_time) {
    return (time() - $communication_start_time) >= $this->timeout;
  }

  /**
   * Sends data compressed by deflate algorithm to the server.
   * 
   * @param string $url_suffix
   *  Url suffix.
   * @param array $parameters
   *  Parameters to be send to server.
   *
   * @return string
   *  Xml response.
   * 
   * @throws Exception
   *  If something went wrong when communication with server.
   */
  protected function doPostRequest($url_suffix, $parameters) {
    $auth = base64_encode("{$this->login}:{$this->password}");
    $params = array(
      'http' => array(
        'method' => 'POST',
        'content' => http_build_query($parameters),
        'timeout' => $this->timeout,
        'header' => "Content-type: application/x-www-form-urlencoded;charset=\"utf-8\"\r\nAuthorization: Basic {$auth}",
      ),
      'ssl' => array(
        'verify_peer' => FALSE,
        'verify_host' => 1,
      ),
    );
    $context = stream_context_create($params);

    $communication_start_time = time();
    $fp = fopen("{$this->serverUrl}{$url_suffix}", 'rb', NULL, $context);
    if (!$fp) {
      $this->throwConnectionException($communication_start_time, t("Problem occurred while connecting to API's server."));
    }
    stream_set_blocking($fp, FALSE);

    $response = $this->readResponse($fp);
    fclose($fp);

    if (empty($response)) {
      $this->throwConnectionException($communication_start_time, t("Problem occurred while reading data from API's server."));
    }

    return $response;
  }

  /**
   * @param int $communication_start_time
   * @param string $message
   * 
   * @throws Exception
   */
  private function throwConnectionException($communication_start_time, $message) {
    $timeout_msg = $this->isTimedOut($communication_start_time) ? t('API server response time exceeded (@timeout)', array('@timeout' => $this->timeout)) : '';
    throw new Exception($message . ' ' . $timeout_msg);
  }

  /**
   * @param mixed $fp
   *  File pointer.
   * @return string
   *  Read string.
   */
  private function readResponse($fp) {
    $response = '';

    while (!feof($fp)) {
      $response .= fgets($fp, self::maxFileReadingStringLength);
    }

    return $response;
  }

}
